package javax.swing;

import javax.accessibility.AccessibleAction;
import javax.accessibility.AccessibleIcon;
import java.awt.event.FocusListener;
import java.awt.IllegalComponentStateException;
import java.awt.FontMetrics;
import java.awt.Cursor;
import javax.accessibility.AccessibleValue;
import javax.accessibility.AccessibleText;
import java.util.Locale;
import javax.accessibility.AccessibleComponent;
import javax.accessibility.AccessibleRole;
import javax.swing.event.ListDataEvent;
import javax.accessibility.AccessibleStateSet;
import javax.accessibility.AccessibleState;
import java.beans.PropertyChangeEvent;
import javax.swing.event.ListDataListener;
import java.beans.PropertyChangeListener;
import javax.accessibility.AccessibleSelection;
import java.io.Serializable;
import javax.accessibility.AccessibleContext;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.awt.Container;
import java.awt.Insets;
import java.util.Collections;
import java.util.ArrayList;
import java.util.List;
import javax.swing.event.ListSelectionEvent;
import sun.awt.AWTAccessor;
import java.awt.event.MouseEvent;
import javax.swing.text.Position;
import sun.swing.SwingUtilities2;
import java.awt.HeadlessException;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.awt.Point;
import java.beans.Transient;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Component;
import javax.swing.plaf.ListUI;
import java.util.Vector;
import javax.swing.event.ListSelectionListener;
import java.awt.Color;
import javax.accessibility.Accessible;

public class JList<E> extends JComponent implements Scrollable, Accessible
{
    private static final String uiClassID = "ListUI";
    public static final int VERTICAL = 0;
    public static final int VERTICAL_WRAP = 1;
    public static final int HORIZONTAL_WRAP = 2;
    private int fixedCellWidth;
    private int fixedCellHeight;
    private int horizontalScrollIncrement;
    private E prototypeCellValue;
    private int visibleRowCount;
    private Color selectionForeground;
    private Color selectionBackground;
    private boolean dragEnabled;
    private ListSelectionModel selectionModel;
    private ListModel<E> dataModel;
    private ListCellRenderer<? super E> cellRenderer;
    private ListSelectionListener selectionListener;
    private int layoutOrientation;
    private DropMode dropMode;
    private transient JList.DropLocation dropLocation;
    
    public JList(ListModel<E> dataModel) {
        this.fixedCellWidth = -1;
        this.fixedCellHeight = -1;
        this.horizontalScrollIncrement = -1;
        this.visibleRowCount = 8;
        this.dropMode = DropMode.USE_SELECTION;
        if (dataModel == null) {
            throw new IllegalArgumentException("dataModel must be non null");
        }
        ToolTipManager.sharedInstance().registerComponent(this);
        this.layoutOrientation = 0;
        this.dataModel = dataModel;
        this.selectionModel = this.createSelectionModel();
        this.setAutoscrolls(true);
        this.setOpaque(true);
        this.updateUI();
        return;
    }
    
    public JList(final E[] array) {
        this(new AbstractListModel<E>() {
            @Override
            public int getSize() {
                return array.length;
            }
            
            @Override
            public E getElementAt(int n) {
                return array[n];
            }
        });
    }
    
    public JList(final Vector<? extends E> vector) {
        this(new AbstractListModel<E>() {
            @Override
            public int getSize() {
                return vector.size();
            }
            
            @Override
            public E getElementAt(int index) {
                return vector.elementAt(index);
            }
        });
    }
    
    public JList() {
        this(new AbstractListModel<E>() {
            @Override
            public int getSize() {
                return 0;
            }
            
            @Override
            public E getElementAt(int n) {
                throw new IndexOutOfBoundsException("No Data Model");
            }
        });
    }
    
    public ListUI getUI() {
        return (ListUI)this.ui;
    }
    
    public void setUI(ListUI ui) {
        super.setUI(ui);
    }
    
    @Override
    public void updateUI() {
        this.setUI((ListUI)UIManager.getUI(this));
        ListCellRenderer<Object> cellRenderer = this.getCellRenderer();
        if ((cellRenderer instanceof Component)) {
            SwingUtilities.updateComponentTreeUI((Component)cellRenderer);
        }
    }
    
    @Override
    public String getUIClassID() {
        return "ListUI";
    }
    
    private void updateFixedCellSize() {
        ListCellRenderer cellRenderer = this.getCellRenderer();
        Object prototypeCellValue = this.getPrototypeCellValue();
        if (cellRenderer != null && prototypeCellValue != null) {
            Component listCellRendererComponent = cellRenderer.getListCellRendererComponent(this, prototypeCellValue, 0, false, false);
            Font font = listCellRendererComponent.getFont();
            listCellRendererComponent.setFont(this.getFont());
            Dimension preferredSize = listCellRendererComponent.getPreferredSize();
            this.fixedCellWidth = preferredSize.width;
            this.fixedCellHeight = preferredSize.height;
            listCellRendererComponent.setFont(font);
        }
    }
    
    public E getPrototypeCellValue() {
        return this.prototypeCellValue;
    }
    
    public void setPrototypeCellValue(E e) {
        Object prototypeCellValue = this.prototypeCellValue;
        this.prototypeCellValue = e;
        if (e != null && !e.equals(prototypeCellValue)) {
            this.updateFixedCellSize();
        }
        this.firePropertyChange("prototypeCellValue", prototypeCellValue, e);
    }
    
    public int getFixedCellWidth() {
        return this.fixedCellWidth;
    }
    
    public void setFixedCellWidth(int fixedCellWidth) {
        this.firePropertyChange("fixedCellWidth", this.fixedCellWidth, this.fixedCellWidth = fixedCellWidth);
    }
    
    public int getFixedCellHeight() {
        return this.fixedCellHeight;
    }
    
    public void setFixedCellHeight(int fixedCellHeight) {
        this.firePropertyChange("fixedCellHeight", this.fixedCellHeight, this.fixedCellHeight = fixedCellHeight);
    }
    
    @Transient
    public ListCellRenderer<? super E> getCellRenderer() {
        return this.cellRenderer;
    }
    
    public void setCellRenderer(ListCellRenderer<? super E> listCellRenderer) {
        ListCellRenderer<Object> cellRenderer = this.cellRenderer;
        this.cellRenderer = listCellRenderer;
        if (listCellRenderer != null && !listCellRenderer.equals(cellRenderer)) {
            this.updateFixedCellSize();
        }
        this.firePropertyChange("cellRenderer", cellRenderer, listCellRenderer);
    }
    
    public Color getSelectionForeground() {
        return this.selectionForeground;
    }
    
    public void setSelectionForeground(Color selectionForeground) {
        this.firePropertyChange("selectionForeground", this.selectionForeground, this.selectionForeground = selectionForeground);
    }
    
    public Color getSelectionBackground() {
        return this.selectionBackground;
    }
    
    public void setSelectionBackground(Color selectionBackground) {
        this.firePropertyChange("selectionBackground", this.selectionBackground, this.selectionBackground = selectionBackground);
    }
    
    public int getVisibleRowCount() {
        return this.visibleRowCount;
    }
    
    public void setVisibleRowCount(int b) {
        int visibleRowCount = this.visibleRowCount;
        this.visibleRowCount = Math.max(0, b);
        this.firePropertyChange("visibleRowCount", visibleRowCount, b);
    }
    
    public int getLayoutOrientation() {
        return this.layoutOrientation;
    }
    
    public void setLayoutOrientation(int layoutOrientation) {
        int layoutOrientation2 = this.layoutOrientation;
        switch (layoutOrientation) {
            case 0:
            case 1:
            case 2: {
                this.firePropertyChange("layoutOrientation", layoutOrientation2, this.layoutOrientation = layoutOrientation);
                return;
            }
            default: {
                throw new IllegalArgumentException("layoutOrientation must be one of: VERTICAL, HORIZONTAL_WRAP or VERTICAL_WRAP");
            }
        }
    }
    
    public int getFirstVisibleIndex() {
        Rectangle visibleRect = this.getVisibleRect();
        int n;
        if (this.getComponentOrientation().isLeftToRight()) {
            n = this.locationToIndex(visibleRect.getLocation());
        }
        else {
            n = this.locationToIndex(new Point(visibleRect.x + visibleRect.width - 1, visibleRect.y));
        }
        if (n != -1) {
            Rectangle cellBounds = this.getCellBounds(n, n);
            if (cellBounds != null) {
                SwingUtilities.computeIntersection(visibleRect.x, visibleRect.y, visibleRect.width, visibleRect.height, cellBounds);
                if (cellBounds.width == 0 || cellBounds.height == 0) {
                    n = -1;
                }
            }
        }
        return n;
    }
    
    public int getLastVisibleIndex() {
        int leftToRight = this.getComponentOrientation().isLeftToRight();
        Rectangle visibleRect = this.getVisibleRect();
        Point point;
        if (!(leftToRight == 0)) {
            point = new Point(visibleRect.x + visibleRect.width - 1, visibleRect.y + visibleRect.height - 1);
        }
        else {
            point = new Point(visibleRect.x, visibleRect.y + visibleRect.height - 1);
        }
        int locationToIndex = this.locationToIndex(point);
        if (locationToIndex != -1) {
            Rectangle cellBounds = this.getCellBounds(locationToIndex, locationToIndex);
            if (cellBounds != null) {
                SwingUtilities.computeIntersection(visibleRect.x, visibleRect.y, visibleRect.width, visibleRect.height, cellBounds);
                if (cellBounds.width == 0 || cellBounds.height == 0) {
                    int n = this.getLayoutOrientation() == 2;
                    Object p = n ? new Point(point.x, visibleRect.y) : new Point(visibleRect.x, point.y);
                    int locationToIndex2 = -1;
                    locationToIndex = -1;
                    do {
                        locationToIndex2 = this.locationToIndex(p);
                        if (locationToIndex2 != -1) {
                            Rectangle cellBounds2 = this.getCellBounds(locationToIndex2, locationToIndex2);
                            if (locationToIndex2 != locationToIndex && cellBounds2 != null && cellBounds2.contains(p)) {
                                locationToIndex = locationToIndex2;
                                if (!(n == 0)) {
                                    p.y = cellBounds2.y + cellBounds2.height;
                                    if (p.y >= point.y) {
                                    }
                                    else {
                                        continue;
                                    }
                                }
                                else {
                                    p.x = cellBounds2.x + cellBounds2.width;
                                    if (p.x >= point.x) {
                                    }
                                    else {
                                        continue;
                                    }
                                }
                            }
                            else {
                            }
                        }
                        else {
                            continue;
                        }
                    } while (locationToIndex2 != -1 && locationToIndex2 != locationToIndex2);
                }
            }
        }
        return locationToIndex;
    }
    
    public void ensureIndexIsVisible(int n) {
        Rectangle cellBounds = this.getCellBounds(n, n);
        if (cellBounds != null) {
            this.scrollRectToVisible(cellBounds);
        }
    }
    
    public void setDragEnabled(boolean dragEnabled) {
        if (dragEnabled && GraphicsEnvironment.isHeadless()) {
            throw new HeadlessException();
        }
        this.dragEnabled = dragEnabled;
        return;
    }
    
    public boolean getDragEnabled() {
        return this.dragEnabled;
    }
    
    public final void setDropMode(DropMode dropMode) {
        if (dropMode != null) {
            switch (dropMode) {
                case USE_SELECTION:
                case ON:
                case INSERT:
                case ON_OR_INSERT: {
                    this.dropMode = dropMode;
                    return;
                }
                default: {
                    break;
                }
            }
        }
        throw new IllegalArgumentException(dropMode + ": Unsupported drop mode for list");
    }
    
    public final DropMode getDropMode() {
        return this.dropMode;
    }
    
    @Override
    JList.DropLocation dropLocationForPoint(Point p) {
        Object o = null;
        Rectangle cellBounds = null;
        int locationToIndex = this.locationToIndex(p);
        if (locationToIndex != -1) {
            cellBounds = this.getCellBounds(locationToIndex, locationToIndex);
        }
        switch (this.dropMode) {
            case USE_SELECTION:
            case ON: {
                o = new JList.DropLocation(p, (cellBounds != null && cellBounds.contains(p)) ? locationToIndex : -1, false);
                break;
            }
            case INSERT: {
                if (locationToIndex == -1) {
                    o = new JList.DropLocation(p, this.getModel().getSize(), true);
                    break;
                }
                if (this.layoutOrientation == 2) {
                    if (SwingUtilities2.liesInHorizontal(cellBounds, p, this.getComponentOrientation().isLeftToRight(), false) == SwingUtilities2.Section.TRAILING) {
                        ++locationToIndex;
                    }
                    else if (locationToIndex == this.getModel().getSize() - 1 && p.y >= cellBounds.y + cellBounds.height) {
                        ++locationToIndex;
                    }
                }
                else if (SwingUtilities2.liesInVertical(cellBounds, p, false) == SwingUtilities2.Section.TRAILING) {
                    ++locationToIndex;
                }
                o = new JList.DropLocation(p, locationToIndex, true);
                break;
            }
            case ON_OR_INSERT: {
                if (locationToIndex == -1) {
                    o = new JList.DropLocation(p, this.getModel().getSize(), true);
                    break;
                }
                boolean n = false;
                if (this.layoutOrientation == 2) {
                    SwingUtilities2.Section liesInHorizontal = SwingUtilities2.liesInHorizontal(cellBounds, p, this.getComponentOrientation().isLeftToRight(), true);
                    if (liesInHorizontal == SwingUtilities2.Section.TRAILING) {
                        ++locationToIndex;
                        n = true;
                    }
                    else if (locationToIndex == this.getModel().getSize() - 1 && p.y >= cellBounds.y + cellBounds.height) {
                        ++locationToIndex;
                        n = true;
                    }
                    else if (liesInHorizontal == SwingUtilities2.Section.LEADING) {
                        n = true;
                    }
                }
                else {
                    SwingUtilities2.Section liesInVertical = SwingUtilities2.liesInVertical(cellBounds, p, true);
                    if (liesInVertical == SwingUtilities2.Section.LEADING) {
                        n = true;
                    }
                    else if (liesInVertical == SwingUtilities2.Section.TRAILING) {
                        ++locationToIndex;
                        n = true;
                    }
                }
                o = new JList.DropLocation(p, locationToIndex, (boolean)n);
                break;
            }
            default: {
                assert false : "Unexpected drop mode";
                break;
            }
        }
        return o;
    }
    
    @Override
    Object setDropLocation(TransferHandler.DropLocation dropLocation, Object o, boolean b) {
        Object o2 = null;
        JList.DropLocation dropLocation2 = (JList.DropLocation)dropLocation;
        if (this.dropMode == DropMode.USE_SELECTION) {
            if (dropLocation2 == null) {
                if (!b && o != null) {
                    this.setSelectedIndices(((int[][])(int[][])o)[0]);
                    SwingUtilities2.setLeadAnchorWithoutSelection(this.getSelectionModel(), ((int[][])(int[][])o)[1][1], ((int[][])(int[][])o)[1][0]);
                }
            }
            else {
                if (this.dropLocation == null) {
                    o2 = new int[][] { this.getSelectedIndices(), { this.getAnchorSelectionIndex(), this.getLeadSelectionIndex() } };
                }
                else {
                    o2 = o;
                }
                int index = dropLocation2.getIndex();
                if (index == -1) {
                    this.clearSelection();
                    this.getSelectionModel().setAnchorSelectionIndex(-1);
                    this.getSelectionModel().setLeadSelectionIndex(-1);
                }
                else {
                    this.setSelectionInterval(index, index);
                }
            }
        }
        this.firePropertyChange("dropLocation", this.dropLocation, this.dropLocation = dropLocation2);
        return o2;
    }
    
    public final JList.DropLocation getDropLocation() {
        return this.dropLocation;
    }
    
    public int getNextMatch(String upperCase, int i, Position.Bias bias) {
        ListModel<E> model = this.getModel();
        int size = model.getSize();
        if (upperCase == null) {
            throw new IllegalArgumentException();
        }
        if (i < 0 || i >= size) {
            throw new IllegalArgumentException();
        }
        upperCase = upperCase.toUpperCase();
        int n = (bias == Position.Bias.Forward) ? 1 : -1;
        do {
            Object element = model.getElementAt(i);
            if (element != null) {
                String s;
                if ((element instanceof String)) {
                    s = ((String)element).toUpperCase();
                }
                else {
                    s = element.toString();
                    if (s != null) {
                        s = s.toUpperCase();
                    }
                }
                if (s != null && s.startsWith(upperCase)) {
                    return i;
                }
            }
            i = (i + n + size) % size;
        } while (i != i);
        return -1;
    }
    
    @Override
    public String getToolTipText(MouseEvent mouseEvent) {
        if (mouseEvent != null) {
            Point point = mouseEvent.getPoint();
            int locationToIndex = this.locationToIndex(point);
            ListCellRenderer cellRenderer = this.getCellRenderer();
            Rectangle cellBounds;
            if (locationToIndex != -1 && cellRenderer != null && (cellBounds = this.getCellBounds(locationToIndex, locationToIndex)) != null && cellBounds.contains(point.x, point.y)) {
                ListSelectionModel selectionModel = this.getSelectionModel();
                Component listCellRendererComponent = cellRenderer.getListCellRendererComponent(this, this.getModel().getElementAt(locationToIndex), locationToIndex, selectionModel.isSelectedIndex(locationToIndex), this.hasFocus() && selectionModel.getLeadSelectionIndex() == locationToIndex);
                if ((listCellRendererComponent instanceof JComponent)) {
                    point.translate(-cellBounds.x, -cellBounds.y);
                    MouseEvent mouseEvent2 = new MouseEvent(listCellRendererComponent, mouseEvent.getID(), mouseEvent.getWhen(), mouseEvent.getModifiers(), point.x, point.y, mouseEvent.getXOnScreen(), mouseEvent.getYOnScreen(), mouseEvent.getClickCount(), mouseEvent.isPopupTrigger(), 0);
                    AWTAccessor.MouseEventAccessor mouseEventAccessor = AWTAccessor.getMouseEventAccessor();
                    mouseEventAccessor.setCausedByTouchEvent(mouseEvent2, mouseEventAccessor.isCausedByTouchEvent(mouseEvent));
                    String toolTipText = ((JComponent)listCellRendererComponent).getToolTipText(mouseEvent2);
                    if (toolTipText != null) {
                        return toolTipText;
                    }
                }
            }
        }
        return super.getToolTipText();
    }
    
    public int locationToIndex(Point point) {
        ListUI ui = this.getUI();
        return (ui != null) ? ui.locationToIndex(this, point) : -1;
    }
    
    public Point indexToLocation(int n) {
        ListUI ui = this.getUI();
        return (ui != null) ? ui.indexToLocation(this, n) : null;
    }
    
    public Rectangle getCellBounds(int n, int n2) {
        ListUI ui = this.getUI();
        return (ui != null) ? ui.getCellBounds(this, n, n2) : null;
    }
    
    public ListModel<E> getModel() {
        return this.dataModel;
    }
    
    public void setModel(ListModel<E> dataModel) {
        if (dataModel == null) {
            throw new IllegalArgumentException("model must be non null");
        }
        this.firePropertyChange("model", this.dataModel, this.dataModel = dataModel);
        this.clearSelection();
        return;
    }
    
    public void setListData(final E[] array) {
        this.setModel(new AbstractListModel<E>() {
            @Override
            public int getSize() {
                return array.length;
            }
            
            @Override
            public E getElementAt(int n) {
                return array[n];
            }
        });
    }
    
    public void setListData(final Vector<? extends E> vector) {
        this.setModel(new AbstractListModel<E>() {
            @Override
            public int getSize() {
                return vector.size();
            }
            
            @Override
            public E getElementAt(int index) {
                return vector.elementAt(index);
            }
        });
    }
    
    protected ListSelectionModel createSelectionModel() {
        return new DefaultListSelectionModel();
    }
    
    public ListSelectionModel getSelectionModel() {
        return this.selectionModel;
    }
    
    protected void fireSelectionValueChanged(int firstIndex, int lastIndex, boolean isAdjusting) {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ListSelectionListener.class) {
                if (o == null) {
                    o = new ListSelectionEvent(this, firstIndex, lastIndex, isAdjusting);
                }
                ((ListSelectionListener)listenerList[i + 1]).valueChanged(o);
            }
        }
    }
    
    public void addListSelectionListener(ListSelectionListener l) {
        if (this.selectionListener == null) {
            this.selectionListener = new JList.ListSelectionHandler();
            this.getSelectionModel().addListSelectionListener(this.selectionListener);
        }
        this.listenerList.add(ListSelectionListener.class, l);
    }
    
    public void removeListSelectionListener(ListSelectionListener l) {
        this.listenerList.remove(ListSelectionListener.class, l);
    }
    
    public ListSelectionListener[] getListSelectionListeners() {
        return (ListSelectionListener[])this.listenerList.getListeners(ListSelectionListener.class);
    }
    
    public void setSelectionModel(ListSelectionModel selectionModel) {
        if (selectionModel == null) {
            throw new IllegalArgumentException("selectionModel must be non null");
        }
        if (this.selectionListener != null) {
            this.selectionModel.removeListSelectionListener(this.selectionListener);
            selectionModel.addListSelectionListener(this.selectionListener);
        }
        this.firePropertyChange("selectionModel", this.selectionModel, this.selectionModel = selectionModel);
        return;
    }
    
    public void setSelectionMode(int selectionMode) {
        this.getSelectionModel().setSelectionMode(selectionMode);
    }
    
    public int getSelectionMode() {
        return this.getSelectionModel().getSelectionMode();
    }
    
    public int getAnchorSelectionIndex() {
        return this.getSelectionModel().getAnchorSelectionIndex();
    }
    
    public int getLeadSelectionIndex() {
        return this.getSelectionModel().getLeadSelectionIndex();
    }
    
    public int getMinSelectionIndex() {
        return this.getSelectionModel().getMinSelectionIndex();
    }
    
    public int getMaxSelectionIndex() {
        return this.getSelectionModel().getMaxSelectionIndex();
    }
    
    public boolean isSelectedIndex(int n) {
        return this.getSelectionModel().isSelectedIndex(n);
    }
    
    public boolean isSelectionEmpty() {
        return this.getSelectionModel().isSelectionEmpty();
    }
    
    public void clearSelection() {
        this.getSelectionModel().clearSelection();
    }
    
    public void setSelectionInterval(int n, int n2) {
        this.getSelectionModel().setSelectionInterval(n, n2);
    }
    
    public void addSelectionInterval(int n, int n2) {
        this.getSelectionModel().addSelectionInterval(n, n2);
    }
    
    public void removeSelectionInterval(int n, int n2) {
        this.getSelectionModel().removeSelectionInterval(n, n2);
    }
    
    public void setValueIsAdjusting(boolean valueIsAdjusting) {
        this.getSelectionModel().setValueIsAdjusting(valueIsAdjusting);
    }
    
    public boolean getValueIsAdjusting() {
        return this.getSelectionModel().getValueIsAdjusting();
    }
    
    @Transient
    public int[] getSelectedIndices() {
        ListSelectionModel selectionModel = this.getSelectionModel();
        int i = selectionModel.getMinSelectionIndex();
        int maxSelectionIndex = selectionModel.getMaxSelectionIndex();
        if (i < 0 || maxSelectionIndex < 0) {
            return new int[0];
        }
        int[] array = new int[1 + (maxSelectionIndex - i)];
        int n = 0;
        for (; i <= maxSelectionIndex; ++i) {
            if (selectionModel.isSelectedIndex(i)) {
                array[n++] = i;
            }
        }
        int[] array2 = new int[n];
        System.arraycopy(array, 0, array2, 0, n);
        return array2;
    }
    
    public void setSelectedIndex(int n) {
        if (n >= this.getModel().getSize()) {
            return;
        }
        this.getSelectionModel().setSelectionInterval(n, n);
        return;
    }
    
    public void setSelectedIndices(int[] array) {
        ListSelectionModel selectionModel = this.getSelectionModel();
        selectionModel.clearSelection();
        int size = this.getModel().getSize();
        for (int n : array) {
            if (n < size) {
                selectionModel.addSelectionInterval(n, n);
            }
        }
    }
    
    @Deprecated
    public Object[] getSelectedValues() {
        ListSelectionModel selectionModel = this.getSelectionModel();
        ListModel<E> model = this.getModel();
        int i = selectionModel.getMinSelectionIndex();
        int maxSelectionIndex = selectionModel.getMaxSelectionIndex();
        if (i < 0 || maxSelectionIndex < 0) {
            return new Object[0];
        }
        Object[] array = new Object[1 + (maxSelectionIndex - i)];
        int n = 0;
        for (; i <= maxSelectionIndex; ++i) {
            if (selectionModel.isSelectedIndex(i)) {
                array[n++] = model.getElementAt(i);
            }
        }
        Object[] array2 = new Object[n];
        System.arraycopy(array, 0, array2, 0, n);
        return array2;
    }
    
    public List<E> getSelectedValuesList() {
        ListSelectionModel selectionModel = this.getSelectionModel();
        ListModel<E> model = this.getModel();
        int i = selectionModel.getMinSelectionIndex();
        int maxSelectionIndex = selectionModel.getMaxSelectionIndex();
        if (i < 0 || maxSelectionIndex < 0) {
            return Collections.emptyList();
        }
        ArrayList<Object> list = new ArrayList();
        for (; i <= maxSelectionIndex; ++i) {
            if (selectionModel.isSelectedIndex(i)) {
                list.add(model.getElementAt(i));
            }
        }
        return list;
    }
    
    public int getSelectedIndex() {
        return this.getMinSelectionIndex();
    }
    
    public E getSelectedValue() {
        int minSelectionIndex = this.getMinSelectionIndex();
        return (minSelectionIndex == -1) ? null : this.getModel().getElementAt(minSelectionIndex);
    }
    
    public void setSelectedValue(Object o, boolean b) {
        if (o == null) {
            this.setSelectedIndex(-1);
        }
        else if (!o.equals(this.getSelectedValue())) {
            ListModel<E> model = this.getModel();
            int i = 0;
            while (i < model.getSize()) {
                if (o.equals(model.getElementAt(i))) {
                    this.setSelectedIndex(i);
                    if (b) {
                        this.ensureIndexIsVisible(i);
                    }
                    this.repaint();
                    return;
                }
                else {
                    ++i;
                }
            }
            this.setSelectedIndex(-1);
        }
        this.repaint();
    }
    
    private void checkScrollableParameters(Rectangle rectangle, int n) {
        if (rectangle == null) {
            throw new IllegalArgumentException("visibleRect must be non-null");
        }
        switch (n) {
            case 0:
            case 1: {
                return;
            }
            default: {
                throw new IllegalArgumentException("orientation must be one of: VERTICAL, HORIZONTAL");
            }
        }
    }
    
    @Override
    public Dimension getPreferredScrollableViewportSize() {
        if (this.getLayoutOrientation() != 0) {
            return this.getPreferredSize();
        }
        Insets insets = this.getInsets();
        int n = insets.left + insets.right;
        int n2 = insets.top + insets.bottom;
        int visibleRowCount = this.getVisibleRowCount();
        int fixedCellWidth = this.getFixedCellWidth();
        int fixedCellHeight = this.getFixedCellHeight();
        if (fixedCellWidth > 0 && fixedCellHeight > 0) {
            return new Dimension(fixedCellWidth + n, visibleRowCount * fixedCellHeight + n2);
        }
        if (this.getModel().getSize() > 0) {
            int width = this.getPreferredSize().width;
            Rectangle cellBounds = this.getCellBounds(0, 0);
            int height;
            if (cellBounds != null) {
                height = visibleRowCount * cellBounds.height + n2;
            }
            else {
                height = 1;
            }
            return new Dimension(width, height);
        }
        else {
            return new Dimension((fixedCellWidth > 0) ? fixedCellWidth : 256, ((fixedCellHeight > 0) ? fixedCellHeight : 16) * visibleRowCount);
        }
    }
    
    @Override
    public int getScrollableUnitIncrement(Rectangle rectangle, int n, int n2) {
        this.checkScrollableParameters(rectangle, n);
        if (n == 1) {
            int locationToIndex2 = this.locationToIndex(rectangle.getLocation());
            if (locationToIndex2 == -1) {
                return 0;
            }
            if (n2 > 0) {
                Rectangle cellBounds4 = this.getCellBounds(locationToIndex2, locationToIndex2);
                return (cellBounds4 == null) ? 0 : (cellBounds4.height - (rectangle.y - cellBounds4.y));
            }
            Rectangle cellBounds2 = this.getCellBounds(locationToIndex2, locationToIndex2);
            if (cellBounds2.y == rectangle.y && locationToIndex2 == 0) {
                return 0;
            }
            if (cellBounds2.y == rectangle.y) {
                Point location2 = cellBounds2.getLocation();
                --location2.y;
                int locationToIndex3 = this.locationToIndex(location2);
                Rectangle cellBounds3 = this.getCellBounds(locationToIndex3, locationToIndex3);
                if (cellBounds3 == null || cellBounds3.y >= cellBounds2.y) {
                    return 0;
                }
                return cellBounds3.height;
            }
            else {
                return rectangle.y - cellBounds2.y;
            }
        }
        else {
            if (n == 0 && this.getLayoutOrientation() != 0) {
                int leftToRight = this.getComponentOrientation().isLeftToRight();
                Point location;
                if (!(leftToRight == 0)) {
                    location = rectangle.getLocation();
                }
                else {
                    location = new Point(rectangle.x + rectangle.width - 1, rectangle.y);
                }
                int locationToIndex = this.locationToIndex(location);
                if (locationToIndex != -1) {
                    Rectangle cellBounds = this.getCellBounds(locationToIndex, locationToIndex);
                    if (cellBounds != null && cellBounds.contains(location)) {
                        int x;
                        int x2;
                        if (!(leftToRight == 0)) {
                            x = rectangle.x;
                            x2 = cellBounds.x;
                        }
                        else {
                            x = rectangle.x + rectangle.width;
                            x2 = cellBounds.x + cellBounds.width;
                        }
                        if (x2 != x) {
                            if (n2 < 0) {
                                return Math.abs(x - x2);
                            }
                            if (!(leftToRight == 0)) {
                                return x2 + cellBounds.width - x;
                            }
                            return x - cellBounds.x;
                        }
                        else {
                            return cellBounds.width;
                        }
                    }
                }
            }
            Font font = this.getFont();
            return (font != null) ? font.getSize() : 1;
        }
    }
    
    @Override
    public int getScrollableBlockIncrement(Rectangle rectangle, int n, int n2) {
        this.checkScrollableParameters(rectangle, n);
        if (n == 1) {
            int n6 = rectangle.height;
            if (n2 > 0) {
                int locationToIndex4 = this.locationToIndex(new Point(rectangle.x, rectangle.y + rectangle.height - 1));
                if (locationToIndex4 != -1) {
                    Rectangle cellBounds5 = this.getCellBounds(locationToIndex4, locationToIndex4);
                    if (cellBounds5 != null) {
                        n6 = cellBounds5.y - rectangle.y;
                        if (n6 == 0 && locationToIndex4 < this.getModel().getSize() - 1) {
                            n6 = cellBounds5.height;
                        }
                    }
                }
            }
            else {
                int locationToIndex3 = this.locationToIndex(new Point(rectangle.x, rectangle.y - rectangle.height));
                int n7 = this.getFirstVisibleIndex();
                if (locationToIndex3 != -1) {
                    if (n7 == -1) {
                        n7 = this.locationToIndex(rectangle.getLocation());
                    }
                    Rectangle rectangle2 = this.getCellBounds(locationToIndex3, locationToIndex3);
                    Rectangle cellBounds3 = this.getCellBounds(n7, n7);
                    if (rectangle2 != null && cellBounds3 != null) {
                        while (rectangle2.y + rectangle.height < cellBounds3.y + cellBounds3.height && rectangle2.y < cellBounds3.y) {
                            ++locationToIndex3;
                            rectangle2 = this.getCellBounds(locationToIndex3, locationToIndex3);
                        }
                        n6 = rectangle.y - rectangle2.y;
                        if (n6 <= 0 && rectangle2.y > 0) {
                            --locationToIndex3;
                            Rectangle cellBounds4 = this.getCellBounds(locationToIndex3, locationToIndex3);
                            if (cellBounds4 != null) {
                                n6 = rectangle.y - cellBounds4.y;
                            }
                        }
                    }
                }
            }
            return n6;
        }
        else if (n == 0 && this.getLayoutOrientation() != 0) {
            int leftToRight = this.getComponentOrientation().isLeftToRight();
            int n3 = rectangle.width;
            if (n2 > 0) {
                int locationToIndex2 = this.locationToIndex(new Point(rectangle.x + (leftToRight ? (rectangle.width - 1) : 0), rectangle.y));
                if (locationToIndex2 != -1) {
                    Rectangle cellBounds2 = this.getCellBounds(locationToIndex2, locationToIndex2);
                    if (cellBounds2 != null) {
                        if (!(leftToRight == 0)) {
                            n3 = cellBounds2.x - rectangle.x;
                        }
                        else {
                            n3 = rectangle.x + rectangle.width - (cellBounds2.x + cellBounds2.width);
                        }
                        if (n3 < 0) {
                            n3 += cellBounds2.width;
                        }
                        else if (n3 == 0 && locationToIndex2 < this.getModel().getSize() - 1) {
                            n3 = cellBounds2.width;
                        }
                    }
                }
            }
            else {
                int locationToIndex = this.locationToIndex(new Point(rectangle.x + (leftToRight ? (-rectangle.width) : (rectangle.width - 1 + rectangle.width)), rectangle.y));
                if (locationToIndex != -1) {
                    Rectangle cellBounds = this.getCellBounds(locationToIndex, locationToIndex);
                    if (cellBounds != null) {
                        int n4 = cellBounds.x + cellBounds.width;
                        if (!(leftToRight == 0)) {
                            if (cellBounds.x < rectangle.x - rectangle.width && n4 < rectangle.x) {
                                n3 = rectangle.x - n4;
                            }
                            else {
                                n3 = rectangle.x - cellBounds.x;
                            }
                        }
                        else {
                            int n5 = rectangle.x + rectangle.width;
                            if (n4 > n5 + rectangle.width && cellBounds.x > n5) {
                                n3 = cellBounds.x - n5;
                            }
                            else {
                                n3 = n4 - n5;
                            }
                        }
                    }
                }
            }
            return n3;
        }
        else {
            return rectangle.width;
        }
    }
    
    @Override
    public boolean getScrollableTracksViewportWidth() {
        if (this.getLayoutOrientation() == 2 && this.getVisibleRowCount() <= 0) {
            return true;
        }
        Container unwrappedParent = SwingUtilities.getUnwrappedParent(this);
        return unwrappedParent instanceof JViewport && unwrappedParent.getWidth() > this.getPreferredSize().width;
    }
    
    @Override
    public boolean getScrollableTracksViewportHeight() {
        if (this.getLayoutOrientation() == 1 && this.getVisibleRowCount() <= 0) {
            return true;
        }
        Container unwrappedParent = SwingUtilities.getUnwrappedParent(this);
        return unwrappedParent instanceof JViewport && unwrappedParent.getHeight() > this.getPreferredSize().height;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("ListUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",fixedCellHeight=" + this.fixedCellHeight + ",fixedCellWidth=" + this.fixedCellWidth + ",horizontalScrollIncrement=" + this.horizontalScrollIncrement + ",selectionBackground=" + ((this.selectionBackground != null) ? this.selectionBackground.toString() : "") + ",selectionForeground=" + ((this.selectionForeground != null) ? this.selectionForeground.toString() : "") + ",visibleRowCount=" + this.visibleRowCount + ",layoutOrientation=" + this.layoutOrientation;
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JList.AccessibleJList();
        }
        return this.accessibleContext;
    }
    
    public static final class DropLocation extends TransferHandler.DropLocation
    {
        private final int index;
        private final boolean isInsert;
        
        private DropLocation(Point point, int index, boolean isInsert) {
            super(point);
            this.index = index;
            this.isInsert = isInsert;
        }
        
        public int getIndex() {
            return this.index;
        }
        
        public boolean isInsert() {
            return this.isInsert;
        }
        
        @Override
        public String toString() {
            return this.getClass().getName() + "[dropPoint=" + this.getDropPoint() + ",index=" + this.index + ",insert=" + this.isInsert + "]";
        }
    }
    
    private class ListSelectionHandler implements ListSelectionListener, Serializable
    {
        @Override
        public void valueChanged(ListSelectionEvent listSelectionEvent) {
            JList.this.fireSelectionValueChanged(listSelectionEvent.getFirstIndex(), listSelectionEvent.getLastIndex(), listSelectionEvent.getValueIsAdjusting());
        }
    }
    
    protected class AccessibleJList extends JComponent.AccessibleJComponent implements AccessibleSelection, PropertyChangeListener, ListSelectionListener, ListDataListener
    {
        int leadSelectionIndex;
        
        public AccessibleJList() {
            JList.this.addPropertyChangeListener(this);
            JList.this.getSelectionModel().addListSelectionListener(this);
            JList.this.getModel().addListDataListener(this);
            this.leadSelectionIndex = JList.this.getLeadSelectionIndex();
        }
        
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            String propertyName = propertyChangeEvent.getPropertyName();
            Object oldValue = propertyChangeEvent.getOldValue();
            Object newValue = propertyChangeEvent.getNewValue();
            if (propertyName.compareTo("model") == 0) {
                if (oldValue != null && oldValue instanceof ListModel) {
                    ((ListModel)oldValue).removeListDataListener(this);
                }
                if (newValue != null && newValue instanceof ListModel) {
                    ((ListModel)newValue).addListDataListener(this);
                }
            }
            else if (propertyName.compareTo("selectionModel") == 0) {
                if (oldValue != null && oldValue instanceof ListSelectionModel) {
                    ((ListSelectionModel)oldValue).removeListSelectionListener(this);
                }
                if (newValue != null && newValue instanceof ListSelectionModel) {
                    ((ListSelectionModel)newValue).addListSelectionListener(this);
                }
                this.firePropertyChange("AccessibleSelection", Boolean.valueOf(false), Boolean.valueOf(true));
            }
        }
        
        @Override
        public void valueChanged(ListSelectionEvent listSelectionEvent) {
            int leadSelectionIndex = this.leadSelectionIndex;
            this.leadSelectionIndex = JList.this.getLeadSelectionIndex();
            if (leadSelectionIndex != this.leadSelectionIndex) {
                this.firePropertyChange("AccessibleActiveDescendant", (leadSelectionIndex >= 0) ? this.getAccessibleChild(leadSelectionIndex) : null, (this.leadSelectionIndex >= 0) ? this.getAccessibleChild(this.leadSelectionIndex) : null);
            }
            this.firePropertyChange("AccessibleVisibleData", Boolean.valueOf(false), Boolean.valueOf(true));
            this.firePropertyChange("AccessibleSelection", Boolean.valueOf(false), Boolean.valueOf(true));
            AccessibleStateSet accessibleStateSet = this.getAccessibleStateSet();
            if (JList.this.getSelectionModel().getSelectionMode() != 0) {
                if (!accessibleStateSet.contains(AccessibleState.MULTISELECTABLE)) {
                    accessibleStateSet.add(AccessibleState.MULTISELECTABLE);
                    this.firePropertyChange("AccessibleState", null, AccessibleState.MULTISELECTABLE);
                }
            }
            else if (accessibleStateSet.contains(AccessibleState.MULTISELECTABLE)) {
                accessibleStateSet.remove(AccessibleState.MULTISELECTABLE);
                this.firePropertyChange("AccessibleState", AccessibleState.MULTISELECTABLE, null);
            }
        }
        
        @Override
        public void intervalAdded(ListDataEvent listDataEvent) {
            this.firePropertyChange("AccessibleVisibleData", Boolean.valueOf(false), Boolean.valueOf(true));
        }
        
        @Override
        public void intervalRemoved(ListDataEvent listDataEvent) {
            this.firePropertyChange("AccessibleVisibleData", Boolean.valueOf(false), Boolean.valueOf(true));
        }
        
        @Override
        public void contentsChanged(ListDataEvent listDataEvent) {
            this.firePropertyChange("AccessibleVisibleData", Boolean.valueOf(false), Boolean.valueOf(true));
        }
        
        @Override
        public AccessibleStateSet getAccessibleStateSet() {
            AccessibleStateSet accessibleStateSet = super.getAccessibleStateSet();
            if (JList.this.selectionModel.getSelectionMode() != 0) {
                accessibleStateSet.add(AccessibleState.MULTISELECTABLE);
                return accessibleStateSet;
            }
            return accessibleStateSet;
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.LIST;
        }
        
        @Override
        public Accessible getAccessibleAt(Point point) {
            int locationToIndex = JList.this.locationToIndex(point);
            if (locationToIndex >= 0) {
                return new JList.AccessibleJList.ActionableAccessibleJListChild(JList.this, locationToIndex);
            }
            return null;
        }
        
        @Override
        public int getAccessibleChildrenCount() {
            return JList.this.getModel().getSize();
        }
        
        @Override
        public Accessible getAccessibleChild(int n) {
            if (n >= JList.this.getModel().getSize()) {
                return null;
            }
            return new JList.AccessibleJList.ActionableAccessibleJListChild(JList.this, n);
        }
        
        @Override
        public AccessibleSelection getAccessibleSelection() {
            return this;
        }
        
        @Override
        public int getAccessibleSelectionCount() {
            return JList.this.getSelectedIndices().length;
        }
        
        @Override
        public Accessible getAccessibleSelection(int n) {
            int accessibleSelectionCount = this.getAccessibleSelectionCount();
            if (n < 0 || n >= accessibleSelectionCount) {
                return null;
            }
            return this.getAccessibleChild(JList.this.getSelectedIndices()[n]);
        }
        
        @Override
        public boolean isAccessibleChildSelected(int n) {
            return JList.this.isSelectedIndex(n);
        }
        
        @Override
        public void addAccessibleSelection(int n) {
            JList.this.addSelectionInterval(n, n);
        }
        
        @Override
        public void removeAccessibleSelection(int n) {
            JList.this.removeSelectionInterval(n, n);
        }
        
        @Override
        public void clearAccessibleSelection() {
            JList.this.clearSelection();
        }
        
        @Override
        public void selectAllAccessibleSelection() {
            JList.this.addSelectionInterval(0, this.getAccessibleChildrenCount() - 1);
        }
        
        protected class AccessibleJListChild extends AccessibleContext implements Accessible, AccessibleComponent
        {
            private JList<E> parent;
            int indexInParent;
            private Component component;
            private AccessibleContext accessibleContext;
            private ListModel<E> listModel;
            private ListCellRenderer<? super E> cellRenderer;
            
            public AccessibleJListChild(JList<E> parent, int indexInParent) {
                this.parent = null;
                this.component = null;
                this.accessibleContext = null;
                this.cellRenderer = null;
                this.setAccessibleParent(this.parent = parent);
                this.indexInParent = indexInParent;
                if (parent != null) {
                    this.listModel = parent.getModel();
                    this.cellRenderer = parent.getCellRenderer();
                }
            }
            
            private Component getCurrentComponent() {
                return this.getComponentAtIndex(this.indexInParent);
            }
            
            AccessibleContext getCurrentAccessibleContext() {
                Component componentAtIndex = this.getComponentAtIndex(this.indexInParent);
                if ((componentAtIndex instanceof Accessible)) {
                    return componentAtIndex.getAccessibleContext();
                }
                return null;
            }
            
            private Component getComponentAtIndex(int n) {
                if (n < 0 || n >= this.listModel.getSize()) {
                    return null;
                }
                if (this.parent != null && this.listModel != null && this.cellRenderer != null) {
                    return this.cellRenderer.getListCellRendererComponent(this.parent, this.listModel.getElementAt(n), n, this.parent.isSelectedIndex(n), this.parent.isFocusOwner() && n == this.parent.getLeadSelectionIndex());
                }
                return null;
            }
            
            @Override
            public AccessibleContext getAccessibleContext() {
                return this;
            }
            
            @Override
            public String getAccessibleName() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getAccessibleName();
                }
                return null;
            }
            
            @Override
            public void setAccessibleName(String accessibleName) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.setAccessibleName(accessibleName);
                }
            }
            
            @Override
            public String getAccessibleDescription() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getAccessibleDescription();
                }
                return null;
            }
            
            @Override
            public void setAccessibleDescription(String accessibleDescription) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.setAccessibleDescription(accessibleDescription);
                }
            }
            
            @Override
            public AccessibleRole getAccessibleRole() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getAccessibleRole();
                }
                return null;
            }
            
            @Override
            public AccessibleStateSet getAccessibleStateSet() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                AccessibleStateSet accessibleStateSet;
                if (currentAccessibleContext != null) {
                    accessibleStateSet = currentAccessibleContext.getAccessibleStateSet();
                }
                else {
                    accessibleStateSet = new AccessibleStateSet();
                }
                accessibleStateSet.add(AccessibleState.SELECTABLE);
                if (this.parent.isFocusOwner() && this.indexInParent == this.parent.getLeadSelectionIndex()) {
                    accessibleStateSet.add(AccessibleState.ACTIVE);
                }
                if (this.parent.isSelectedIndex(this.indexInParent)) {
                    accessibleStateSet.add(AccessibleState.SELECTED);
                }
                if (this.isShowing()) {
                    accessibleStateSet.add(AccessibleState.SHOWING);
                }
                else if (accessibleStateSet.contains(AccessibleState.SHOWING)) {
                    accessibleStateSet.remove(AccessibleState.SHOWING);
                }
                if (this.isVisible()) {
                    accessibleStateSet.add(AccessibleState.VISIBLE);
                }
                else if (accessibleStateSet.contains(AccessibleState.VISIBLE)) {
                    accessibleStateSet.remove(AccessibleState.VISIBLE);
                }
                accessibleStateSet.add(AccessibleState.TRANSIENT);
                return accessibleStateSet;
            }
            
            @Override
            public int getAccessibleIndexInParent() {
                return this.indexInParent;
            }
            
            @Override
            public int getAccessibleChildrenCount() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getAccessibleChildrenCount();
                }
                return 0;
            }
            
            @Override
            public Accessible getAccessibleChild(int n) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    Accessible accessibleChild = currentAccessibleContext.getAccessibleChild(n);
                    currentAccessibleContext.setAccessibleParent(this);
                    return accessibleChild;
                }
                return null;
            }
            
            @Override
            public Locale getLocale() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getLocale();
                }
                return null;
            }
            
            @Override
            public void addPropertyChangeListener(PropertyChangeListener listener) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.addPropertyChangeListener(listener);
                }
            }
            
            @Override
            public void removePropertyChangeListener(PropertyChangeListener listener) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.removePropertyChangeListener(listener);
                }
            }
            
            @Override
            public AccessibleComponent getAccessibleComponent() {
                return this;
            }
            
            @Override
            public AccessibleSelection getAccessibleSelection() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                return (currentAccessibleContext != null) ? currentAccessibleContext.getAccessibleSelection() : null;
            }
            
            @Override
            public AccessibleText getAccessibleText() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                return (currentAccessibleContext != null) ? currentAccessibleContext.getAccessibleText() : null;
            }
            
            @Override
            public AccessibleValue getAccessibleValue() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                return (currentAccessibleContext != null) ? currentAccessibleContext.getAccessibleValue() : null;
            }
            
            @Override
            public Color getBackground() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getBackground();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getBackground();
                }
                return null;
            }
            
            @Override
            public void setBackground(Color color) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setBackground(color);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setBackground(color);
                    }
                }
            }
            
            @Override
            public Color getForeground() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getForeground();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getForeground();
                }
                return null;
            }
            
            @Override
            public void setForeground(Color color) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setForeground(color);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setForeground(color);
                    }
                }
            }
            
            @Override
            public Cursor getCursor() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getCursor();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getCursor();
                }
                Accessible accessibleParent = this.getAccessibleParent();
                if ((accessibleParent instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)accessibleParent).getCursor();
                }
                return null;
            }
            
            @Override
            public void setCursor(Cursor cursor) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setCursor(cursor);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setCursor(cursor);
                    }
                }
            }
            
            @Override
            public Font getFont() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getFont();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getFont();
                }
                return null;
            }
            
            @Override
            public void setFont(Font font) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setFont(font);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setFont(font);
                    }
                }
            }
            
            @Override
            public FontMetrics getFontMetrics(Font font) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getFontMetrics(font);
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getFontMetrics(font);
                }
                return null;
            }
            
            @Override
            public boolean isEnabled() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).isEnabled();
                }
                Component currentComponent = this.getCurrentComponent();
                return currentComponent != null && currentComponent.isEnabled();
            }
            
            @Override
            public void setEnabled(boolean b) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setEnabled(b);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setEnabled(b);
                    }
                }
            }
            
            @Override
            public boolean isVisible() {
                int firstVisibleIndex = this.parent.getFirstVisibleIndex();
                int lastVisibleIndex = this.parent.getLastVisibleIndex();
                if (lastVisibleIndex == -1) {
                    lastVisibleIndex = this.parent.getModel().getSize() - 1;
                }
                return this.indexInParent >= firstVisibleIndex && this.indexInParent <= lastVisibleIndex;
            }
            
            @Override
            public void setVisible(boolean b) {
            }
            
            @Override
            public boolean isShowing() {
                return this.parent.isShowing() && this.isVisible();
            }
            
            @Override
            public boolean contains(Point p) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getBounds().contains(p);
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getBounds().contains(p);
                }
                return this.getBounds().contains(p);
            }
            
            @Override
            public Point getLocationOnScreen() {
                if (this.parent != null) {
                    Point locationOnScreen;
                    try {
                        locationOnScreen = this.parent.getLocationOnScreen();
                    }
                    catch (IllegalComponentStateException ex) {
                        return null;
                    }
                    Point indexToLocation = this.parent.indexToLocation(this.indexInParent);
                    if (indexToLocation != null) {
                        indexToLocation.translate(locationOnScreen.x, locationOnScreen.y);
                        return indexToLocation;
                    }
                    return null;
                }
                else {
                    return null;
                }
            }
            
            @Override
            public Point getLocation() {
                if (this.parent != null) {
                    return this.parent.indexToLocation(this.indexInParent);
                }
                return null;
            }
            
            @Override
            public void setLocation(Point p) {
                if (this.parent != null && this.parent.contains(p)) {
                    JList.this.ensureIndexIsVisible(this.indexInParent);
                }
            }
            
            @Override
            public Rectangle getBounds() {
                if (this.parent != null) {
                    return this.parent.getCellBounds(this.indexInParent, this.indexInParent);
                }
                return null;
            }
            
            @Override
            public void setBounds(Rectangle bounds) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setBounds(bounds);
                }
            }
            
            @Override
            public Dimension getSize() {
                Rectangle bounds = this.getBounds();
                if (bounds != null) {
                    return bounds.getSize();
                }
                return null;
            }
            
            @Override
            public void setSize(Dimension dimension) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setSize(dimension);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setSize(dimension);
                    }
                }
            }
            
            @Override
            public Accessible getAccessibleAt(Point point) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getAccessibleAt(point);
                }
                return null;
            }
            
            @Override
            public boolean isFocusTraversable() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).isFocusTraversable();
                }
                Component currentComponent = this.getCurrentComponent();
                return currentComponent != null && currentComponent.isFocusTraversable();
            }
            
            @Override
            public void requestFocus() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).requestFocus();
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.requestFocus();
                    }
                }
            }
            
            @Override
            public void addFocusListener(FocusListener l) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).addFocusListener(l);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.addFocusListener(l);
                    }
                }
            }
            
            @Override
            public void removeFocusListener(FocusListener l) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).removeFocusListener(l);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.removeFocusListener(l);
                    }
                }
            }
            
            @Override
            public AccessibleIcon[] getAccessibleIcon() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getAccessibleIcon();
                }
                return null;
            }
        }
        
        private class ActionableAccessibleJListChild extends JList.AccessibleJList.AccessibleJListChild implements AccessibleAction
        {
            ActionableAccessibleJListChild(JList<E> list, int n) {
                list.super(n);
            }
            
            @Override
            public AccessibleAction getAccessibleAction() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext == null) {
                    return null;
                }
                AccessibleAction accessibleAction = currentAccessibleContext.getAccessibleAction();
                if (accessibleAction != null) {
                    return accessibleAction;
                }
                return this;
            }
            
            @Override
            public boolean doAccessibleAction(int n) {
                if (n == 0) {
                    JList.this.setSelectedIndex(this.indexInParent);
                    return true;
                }
                return false;
            }
            
            @Override
            public String getAccessibleActionDescription(int n) {
                if (n == 0) {
                    return UIManager.getString("AbstractButton.clickText");
                }
                return null;
            }
            
            @Override
            public int getAccessibleActionCount() {
                return 1;
            }
        }
    }
}
